
// global values used throughout
// these may be overridden at runtime by a debug file
// see readDebugFile() below for more information
var BASE_URL = "https://api.constantcontact.com";
var BASE_URL_INSECURE = "http://api.constantcontact.com";
var BASE_LIST_URL = "https://api.constantcontact.com/ws/customers/";
var BASE_SAVE_URL = "https://api.constantcontact.com/ws/customers/";
var API_KEY = "4e0981b1-4cbf-4712-a677-2c6998c7bac1";

// class definition for APIComm class
function APIComm() {
	
	// initialize debug values (if any)
	var dbgSettings = readDebugFile();
	if (dbgSettings) {
		try {
			dbgSettings = eval('(' + dbgSettings + ')');
			if (dbgSettings.BASE_URL)
				BASE_URL = dbgSettings.BASE_URL;
			if (dbgSettings.BASE_URL_INSECURE)
				BASE_URL_INSECURE = dbgSettings.BASE_URL_INSECURE;
			if (dbgSettings.BASE_LIST_URL)
				BASE_LIST_URL = dbgSettings.BASE_LIST_URL;
			if (dbgSettings.BASE_SAVE_URL)
				BASE_SAVE_URL = dbgSettings.BASE_SAVE_URL;
			if (dbgSettings.API_KEY)
				API_KEY = dbgSettings.API_KEY;
		} catch(e) {
			alert("Error reading debug file: " + e.message);
		}
	}


// read debug.cfg in the install directory
// e.g. C:\Program Files\Constant Contact\Contact Capture Tool
// format is JSON
// e.g. {BASE_LIST_URL:"https://api.constantcontact.com/ws/customers/",BASE_SAVE_URL:"https://api.constantcontact.com/ws/customers/",}
function readDebugFile() {
	var fileName = 'debug.cfg';
	var root = air.File.applicationDirectory;
	var fs = null;
	var fileCnt = null;
	try {
		var file = root.resolvePath(fileName);
		if (file && file.exists) {
			fs = new air.FileStream();
			fs.open(file, 'read');
			fileCnt = fs.readUTFBytes(fs.bytesAvailable);
			fs.close();
		}
	} catch(e) {
		air.trace(e.message);
		if(fs != null) {fs.close()};
	}
	return fileCnt;				
}

// return a header for use in all API interactions
// this header contains the username and password of the constant contact user
function getAuthHeader(user_name, password) {
	return new air.URLRequestHeader("Authorization", "Basic " + btoa(API_KEY + '%' + user_name + ':' + password));
}

// convert reserved character entities for xml encoding
function escapeEntities(str) {
	try {
		str = str.split("&").join("&amp;");
		str = str.split("<").join("&lt;");
		str = str.split(">").join("&gt;");
	} catch(e) {}

	return str;
}

// private function to load a url. this function is the workhorse of the APIComm class
// cbfn is a callback function that takes (status, data)
// header can be generated by getAuthHeader()
// set postData to non-null to use http POST method
// set putFlag to true and use postData to use http PUT method
function loadUrl(url, cbfn, header, postData, putFlag) {
    var serviceLoader = new air.URLLoader();
    var serviceRequest = new air.URLRequest(url);

	// handlers
	var handleComplete = function(evt) {  // called for all 2xx statuses
		cbfn(lastStatus, evt.target.data);
	}
	var handleStatus = function(evt) {
		lastStatus = evt.status;  // save for handleComplete (if this is a 2xx status)
		if (evt.status >= 300 || evt.status < 200)
			cbfn(evt.status, "");
	}

	// set the status and completion event handlers
    serviceLoader.addEventListener(air.Event.COMPLETE, handleComplete);
    serviceLoader.addEventListener(air.HTTPStatusEvent.HTTP_STATUS, handleStatus);

	// add header if requested
	if (header)
		serviceRequest.requestHeaders.push(header);
		
	// suppress windows authentication dialog
	serviceRequest.authenticate = false;

	// doing a POST?
	if (postData) {
		if (putFlag)
			serviceRequest.method = "PUT";
		else
			serviceRequest.method = "POST";
		serviceRequest.contentType = "application/atom+xml";
		serviceRequest.requestHeaders.push(new air.URLRequestHeader("Connection", "close"));
		serviceRequest.data = postData;
	}

	// load the url
	serviceLoader.load(serviceRequest);	
}

// update a contact
// email is the email address of the contact
// listsAry is an array of contact list ids
// extraFlds is an array of custom fields
// cbfn is a callback function that takes (status, data)
// set replaceLists to true to replace all lists in the updated contact with those in listsAry
// if replaceLists is false, lists will be merged
this.updateContact = function(user_name, password, email, listsAry, extraFlds, cbfn, replaceLists) {
	// skip if username or password are blank
	if (/^\s*$/.exec(user_name) || /^\s*$/.exec(password))
		return false;

	// query contact first
	var queryUri = BASE_LIST_URL + encodeURIComponent(user_name) + '/contacts?email=' + encodeURIComponent(email);

	var queryContactHandler = function(status, data) {
		if (status != 200)
			return;
		var parser = new DOMParser();
		var xmlDoc = parser.parseFromString(data, "text/xml");
		// check for parse error
		if (xmlDoc.getElementsByTagName("parsererror").length) {
			cbfn(1001, '');  // special parse error status
			return;
		}

		// find edit link to get edit url and id
		var links = xmlDoc.getElementsByTagName('link');
		var editUrl = null;
		var contactId = null;
		for (var i=0; i<links.length; ++i) {
			var rel = links[i].getAttribute('rel');
			if (!rel || rel != 'edit')
				continue;
			editUrl = BASE_URL + links[i].getAttribute('href');
			contactId = BASE_URL_INSECURE + links[i].getAttribute('href');
		}
		if (!editUrl)
			return;

		// now get the contact
		var getContactHandler = function(status, data) {
			if (status != 200)
				return;
			var parser = new DOMParser();
			var xmlDoc = parser.parseFromString(data, "text/xml");
			// check for parse error
			if (xmlDoc.getElementsByTagName("parsererror").length) {
				cbfn(1001, '');  // special parse error status
				return;
			}

			// get entry node
			var entries = xmlDoc.getElementsByTagName('entry');
			if (!entries || entries.length == 0)
				return;
			var entry = entries[0];

			// get Contact node
			var Contacts = entry.getElementsByTagName('Contact');
			if (!Contacts || Contacts.length == 0)
				return;
			var Contact = Contacts[0];

			// replace or add any data to the contact
			for (var i=0; i<extraFlds.length; ++i) {
				if (Contact.getElementsByTagName(extraFlds[i].name).length > 0) {  // replace existing
					if (Contact.getElementsByTagName(extraFlds[i].name)[0].childNodes.length)
						Contact.getElementsByTagName(extraFlds[i].name)[0].childNodes[0].nodeValue = extraFlds[i].value;
					else
						Contact.getElementsByTagName(extraFlds[i].name)[0].appendChild(xmlDoc.createTextNode(extraFlds[i].value));
				} else {  // add new
					var node = xmlDoc.createElement(extraFlds[i].name);
					node.appendChild(xmlDoc.createTextNode(extraFlds[i].value));
					Contact.appendChild(node);
				}
			}

			// see if CountryCode is being set. If so, CountryName needs to be removed
			for (var i=0; i<extraFlds.length; ++i) {
				if (extraFlds[i].name == 'CountryCode') {
					var CountryName = Contact.getElementsByTagName('CountryName')[0];
					if (CountryName)
						Contact.removeChild(CountryName);
					break;
				}
			}

			// see if StateCode is being set. If so, StateName needs to be empty
			for (var i=0; i<extraFlds.length; ++i) {
				if (extraFlds[i].name == 'StateCode') {
					var StateName = Contact.getElementsByTagName('StateName')[0];
					if (StateName)
						Contact.removeChild(StateName);
					// add a blank StateName node
					var node = xmlDoc.createElement('StateName');
					node.appendChild(xmlDoc.createTextNode(''));
					Contact.appendChild(node);
					break;
				}
			}

			// see if StateName is being set. If so, StateCode needs to be empty
			for (var i=0; i<extraFlds.length; ++i) {
				if (extraFlds[i].name == 'StateName') {
					var StateCode = Contact.getElementsByTagName('StateCode')[0];
					if (StateCode)
						Contact.removeChild(StateCode);
					// add a blank StateCode node
					var node = xmlDoc.createElement('StateCode');
					node.appendChild(xmlDoc.createTextNode(''));
					Contact.appendChild(node);
					break;
				}
			}

			// get country and state codes that will be pushed back to the server
			var stateCode;
			if (Contact.getElementsByTagName('StateCode')) {
				var StateCodeNode = Contact.getElementsByTagName('StateCode')[0];
				if (StateCodeNode && StateCodeNode.childNodes && StateCodeNode.childNodes.length > 0)
					stateCode = StateCodeNode.childNodes[0].nodeValue;
			}
			var countryCode;
			if (Contact.getElementsByTagName('CountryCode')) {
				var CountryCodeNode = Contact.getElementsByTagName('CountryCode')[0];
				if (CountryCodeNode && CountryCodeNode.childNodes && CountryCodeNode.childNodes.length > 0)
					countryCode = CountryCodeNode.childNodes[0].nodeValue;
			}

			// validate state/country
			if (stateCode && (stateCode == 'AB' || stateCode == 'BC' || stateCode == 'MB' || stateCode == 'NB' || stateCode == 'NL' || stateCode == 'NT' ||
				stateCode == 'NS' || stateCode == 'NU' || stateCode == 'ON' || stateCode == 'PE' || stateCode == 'QC' || stateCode == 'SK' || stateCode == 'YT')) {
				if (countryCode && countryCode != 'ca') {
					cbfn(1000, "Error: State/province and country do not match existing data. Update failed.");  // special status return
					return;
				}
			} else if (stateCode && countryCode && countryCode != 'us') {
				cbfn(1000, "Error: State/province and country do not match existing data. Update failed.");  // special status return
				return;
			}
	
			// find or create ContactLists node
			var ContactLists = Contact.getElementsByTagName('ContactLists');
			if (!ContactLists || ContactLists.length == 0) {
				ContactLists = xmlDoc.createElement('ContactLists');
				Contact.appendChild(ContactLists);
			} else
				ContactLists = ContactLists[0];

			// remove existing lists if replacing
			if (replaceLists) {
				while (ContactLists.firstChild)
					ContactLists.removeChild(ContactLists.firstChild);
			}

			// add new lists to entry
			for (var i=0; i<listsAry.length; ++i) {
				// see if list already exists
				var addOk = true;
				var lists = ContactLists.getElementsByTagName('ContactList');
				for (var j=0; j<lists.length; ++j) {
					if (lists[j].getAttribute('id') == listsAry[i])
						addOk = false;
				}

				// add the list if it isn't already there
				if (addOk) {
					var node = xmlDoc.createElement('ContactList');
					node.setAttribute('id', escapeEntities(listsAry[i]));
					node.appendChild(xmlDoc.createTextNode(''));
					ContactLists.appendChild(node);
				}
			}

			// serialize entry
			var serializer = new XMLSerializer();
			var updateXML = serializer.serializeToString(entry);

			// update contact
			loadUrl(editUrl, cbfn, getAuthHeader(user_name, password), updateXML, true);
		}  // getContactHandler()
			
		// get contact
		loadUrl(editUrl, getContactHandler, getAuthHeader(user_name, password));
	}  // queryContactHandler()

	// query contact
	loadUrl(queryUri, queryContactHandler, getAuthHeader(user_name, password));
	
	return true;
}

// load contact lists for a constant contact account
// cbfn is a callback function that takes (status, data)
this.loadLists = function(user_name, password, cbfn) {
	// skip if username or password are blank
	if (/^\s*$/.exec(user_name) || /^\s*$/.exec(password))
		return false;

	var uri = BASE_LIST_URL + encodeURIComponent(user_name) + '/lists';

	var loadListsHandler = function(status, data) {
		if (status == 200) {
			var parser = new DOMParser();
			var xmlDoc = parser.parseFromString(data, "text/xml");
			// check for parse error
			if (xmlDoc.getElementsByTagName("parsererror").length) {
				cbfn(1001, '');  // special parse error status
				return;
			}

			var links = xmlDoc.getElementsByTagName('link');
			for (var i=0; i<links.length; ++i) {
				var rel = links[i].getAttribute('rel');
				if (!rel || rel != 'next')
					continue;
				var l = BASE_URL + links[i].getAttribute('href');
				cbfn(status, data);
				loadUrl(l, loadListsHandler, getAuthHeader(user_name, password));
				return;
			}
			// no 'next' link found
			cbfn(status, data);
			cbfn(1000, '');
			return;
		}
		cbfn(status, data);
	}

	loadUrl(uri, loadListsHandler, getAuthHeader(user_name, password));
	
	return true;
}

// add a contact
// emailAddress is the email address of the contact
// listsAry is an array of contact list ids
// extraFlds is an array of custom fields
// cbfn is a callback function that takes (status, data)
this.addContact = function(emailAddress, listsAry, extraFlds, cbfn) {
    var user_name = document.getElementById('userName').value;
    var password = document.getElementById('password').value;

	// skip if username or password are blank
	if (/^\s*$/.exec(user_name) || /^\s*$/.exec(password))
		return false;

	var uri = BASE_SAVE_URL + encodeURIComponent(user_name) + '/contacts';

	var createXML = "<?xml version='1.0' encoding='UTF-8'?>" +
					"<entry xmlns=\"http://www.w3.org/2005/Atom\">" +
					"<title type=\"text\"></title>" +
					"<updated>2008-06-25T21:01:05.227Z</updated>" +
					"<author></author>" +
					"<id>data:,foobar</id>" +
					"<summary type=\"text\">Customer document</summary>" +
					"<content type=\"application/vnd.ctct+xml\">" +
						"<Contact xmlns=\"http://ws.constantcontact.com/ns/1.0/\">" +
						"<EmailAddress>" + emailAddress.toLowerCase() + "</EmailAddress>";

	for (var i=0; i<extraFlds.length; ++i) {
		createXML += '<' + extraFlds[i].name + '>' + escapeEntities(extraFlds[i].value) + '</' + extraFlds[i].name + '>';
	}

	createXML += "<OptInSource>ACTION_BY_CUSTOMER</OptInSource>" +
						"<ContactLists>";

	for (var i=0; i<listsAry.length; ++i)
		createXML += "<ContactList id=\"" + escapeEntities(listsAry[i]) + "\" />";

	createXML += "</ContactLists></Contact></content></entry>";

	loadUrl(uri, cbfn, getAuthHeader(user_name, password), createXML);

	return true;
}


}