/**
 * CLASS
 * 		Main
 * DESCRIPTION
 * 		Main class of the Contact Capture Tool application.
 * 
 * USAGE
 * 		var main = Main.getDefault();
 * 		main.run();
 * @class
 * @public 
 */
Main = function() {
	throw(new Error(
			"You cannot instantiate the 'Main' class. " +
			"Instead, use 'Main.getDefault()' to retrieve the " +
			"class' unique instance."
	));
}
Main.getDefault = function() {
	var context = arguments.callee;
	if (context.instance) { return context.instance };
	function _main() {

		// for update checking
		var appUpdater;

		// global APIComm instance
		var comm;

		/**
		 * The shared PersistenceProvider instance.
		 * @field
		 * @private
		 */
		var persistenceProvider;

		/**
		 * The shared EventManager instance.
		 * @field
		 * @private
		 */
		var eventManager;

		/**
		 * This will hold the settings native window instance
		 */
		var settingsWindow;

		/**
		 * This will hold the EULA native window instance
		 */
		var eulaWindow;

		/**
		 * This will hold the list selection native window instance
		 */
		var listSelWindow;
		
		/**
		 * This will hold the about native window instance
		 */
		var aboutWindow;
		
		/**
		 * Enhances the application UI.
		 * @method
		 * @private
		 */
		function enhanceUI() {
			enhanceButtons ();
		}
		
		/**
		 * Flag we raise when somebody calls 'appExit()' -- this prevents having
		 * it called a second time, and throw.
		 * @field
		 * @private
		 */
		var appExiting;

		// clear the UI fields after a contact is added or updated successfully
		function clearFields() {
			document.getElementById('emailInput').value = '';
			document.getElementById('field1').value = '';
			document.getElementById('field2').value = '';
			document.getElementById('field3').value = '';
			document.getElementById('field4').value = '';
			document.getElementById('field1_state_edit').value = '';
			document.getElementById('field1_state_id').value = -1;
			document.getElementById('field2_state_edit').value = '';
			document.getElementById('field2_state_id').value = -1;
			document.getElementById('field3_state_edit').value = '';
			document.getElementById('field3_state_id').value = -1;
			document.getElementById('field4_state_edit').value = '';
			document.getElementById('field4_state_id').value = -1;
			document.getElementById('field1_country_edit').value = '';
			document.getElementById('field1_country_id').value = -1;
			document.getElementById('field2_country_edit').value = '';
			document.getElementById('field2_country_id').value = -1;
			document.getElementById('field3_country_edit').value = '';
			document.getElementById('field3_country_id').value = -1;
			document.getElementById('field4_country_edit').value = '';
			document.getElementById('field4_country_id').value = -1;
			document.getElementById('field1_emailtype_edit').value = '';
			document.getElementById('field1_emailtype_id').value = -1;
			document.getElementById('field2_emailtype_edit').value = '';
			document.getElementById('field2_emailtype_id').value = -1;
			document.getElementById('field3_emailtype_edit').value = '';
			document.getElementById('field3_emailtype_id').value = -1;
			document.getElementById('field4_emailtype_edit').value = '';
			document.getElementById('field4_emailtype_id').value = -1;
		}

		// enable or disable the ADD button by setting the image
		function enableAddBtn(en) {
			var addBtn = document.getElementById('addButton');
			if (en)
				addBtn.style.backgroundImage = 'url(src/images/add-button.gif)';
			else
				addBtn.style.backgroundImage = 'url(src/images/add-button-disabled.gif)';
		}

		// prompt the user asking them whether they want to update a contact
		// initiates the update process if they choose
		function promptForUpdate(user_name, password, email, listsAry, extraFlds) {
			var options = new air.NativeWindowInitOptions();
			var modalWin = null;
			var width = 420;
			var height = 290;
			var bounds = new air.Rectangle(
			    (air.Capabilities.screenResolutionX - width) / 2,
			  (air.Capabilities.screenResolutionY - height) / 2,
			  width, height);

			options.type = "utility";
			options.resizable = false;

			modalWin = air.HTMLLoader.createRootWindow(true, options, true, bounds);

			modalWin.addEventListener(air.Event.COMPLETE, function (e) {e.target.window.makeWindowModal()} );

			modalWin.load(new air.URLRequest("updateprompt.html"));
			modalWin.window.opener = window;
			modalOpen = true;

			// default to cancel status in case window is closed from title bar
			document.getElementById('updatePromptStatus').value = 'cancel';

			var promptCloseHandler = function() {
				modalOpen = false;
				if (document.getElementById('updatePromptStatus').value == 'cancel') {
					setStatus(document, 'Enter an email address and press Add.', 'black');
//					setResult(document, "Contact exists.  Contact not added.", 'red');
					enableAddBtn(true);  // re-enable add button
					return;
				}

				var replaceLists = (document.getElementById('updatePromptStatus').value == 'replace' ? true : false);

				var updateHandler = function(status, data) {
					enableAddBtn(true);  // re-enable add button
					if (status == 204) {
						clearFields();
						setStatus(document, 'Enter an email address and press Add.', 'black');
						setResult(document, "Contact updated!", 'green');
					} else if (status == 1000) {
						// special status return for state/country code issues
						setStatus(document, data, 'red');
						setResult(document, '', '');
						alert(data);
					} else if (status == 1001) {  // special parse error status
						setStatus(document, "Error parsing the data returned by the server. Please contact support.", 'red');
						setResult(document, '', '');
					} else {
						setStatus(document, "Error: Unexpected error " + status, 'red');
						setResult(document, '', '');
					}
				}
				setStatus(document, "Please wait while your contact updates...", 'black');
				setResult(document, '', '');
				comm.updateContact(user_name, password, email, listsAry, extraFlds, updateHandler, replaceLists);
			}

			modalWin.window.nativeWindow.addEventListener('close', promptCloseHandler, false);
		}

		// main add contact function
		// called when the user clicks the ADD button
		function addEmailAddress() {
			enableAddBtn(false);
			checkPasswordExpiration();

			// check perm conf and username/password
			if (!checkSettings(false)) {
				showSettings();
				enableAddBtn(true);
				return;
			}

			var emailAddress = document.getElementById('emailInput').value;
			emailAddress = emailAddress.replace(/^\s+/, '').replace(/\s+$/, '');  // remove leading/trailing spaces
			var user_name = document.getElementById('userName').value;
			var password = document.getElementById('password').value;

			// check email address
			if (typeof(emailAddress) == 'undefined' || emailAddress == null || /^\s*$/.exec(emailAddress)) {
				setStatus(document, "Please provide an email address.", 'black');
				setResult(document, '', '');
				enableAddBtn(true);
				return;
			}

			// check that at least one contact list is selected
			var listsel = document.getElementById('listInput');
			if (listsel.selectedIndex == -1) {
				setStatus(document, "You need to select a Contact List", 'black');
				setResult(document, '', '');
				enableAddBtn(true);
				return;
			}

			// get list of selected lists
			var listsAry = new Array();
			for (var i=0; i<listsel.options.length; ++i) {
				if (listsel.options[i].selected)
					listsAry.push(listsel.options[i].value);
			}

			// get extra fields
			var stateCode;
			var countryCode;
			var emailType;
			var extraFlds = new Array();
			for (var i=1; i<=4; ++i) {
				var fld = document.getElementById('field' + i);
				var name = document.getElementById('field' + i + '_type_id').value;
				if (name == 'StateCode') {  // use dropdown value instead
					var fld = document.getElementById('field' + i + '_state_id');
					if (fld.value != -1)
						extraFlds.push({"name":name, "value":fld.value});
					stateCode = fld.value;
				} else if (name == 'CountryCode') {  // use dropdown value instead
					var fld = document.getElementById('field' + i + '_country_id');
					if (fld.value != -1)
						extraFlds.push({"name":name, "value":fld.value});
					countryCode = fld.value;
				} else if (name == 'EmailType') {  // use dropdown value instead
					var fld = document.getElementById('field' + i + '_emailtype_id');
					if (fld.value != -1)
						extraFlds.push({"name":name, "value":fld.value});
					emailType = fld.value;
				} else if (name != -1 && fld.value.length > 0) {
					if (fld.value.length > 50) {
						setStatus(document, "The <" + name + "> you entered is too long.", 'red');
						setResult(document, '', '');
						return;
					}
					extraFlds.push({"name":name, "value":fld.value.replace(/<[^>]*>/g, '')});  // remove anything that looks like <xxx>
					//extraFlds.push({"name":name, "value":fld.value});
				}
			}

			// validate state/country
			if (stateCode && (stateCode == 'AB' || stateCode == 'BC' || stateCode == 'MB' || stateCode == 'NB' || stateCode == 'NL' || stateCode == 'NT' ||
				stateCode == 'NS' || stateCode == 'NU' || stateCode == 'ON' || stateCode == 'PE' || stateCode == 'QC' || stateCode == 'SK' || stateCode == 'YT')) {
				if (countryCode && countryCode != -1 && countryCode != 'ca') {
					setStatus(document, "The state/province and country must match. Please try again.", 'red');
					setResult(document, '', '');
					enableAddBtn(true);
					return;
				}
			} else if (stateCode && stateCode != -1 && countryCode && countryCode != -1 && countryCode != 'us') {
				setStatus(document, "The state/province and country must match. Please try again.", 'red');
				setResult(document, '', '');
				enableAddBtn(true);
				return;
			}
				
			// status handler fn
			var addContactHandler = function(status, data) {
				// re-enable add button if not prompting to update
				if (status != 409)
					enableAddBtn(true);

				var lines = data.split('\n');
				if (status == 201) {
					clearFields();
					setStatus(document, 'Enter an email address and click Add.', 'black');
					setResult(document, "Contact added!", 'green');
			    } else if (status == 500) {
					setStatus(document, "Error: " + lines[0] + lines[1], 'red');
					setResult(document, '', '');
			    } else if (status == 400) {
					setStatus(document, "The email address or data you entered is not valid.", 'red');
					setResult(document, '', '');
			    } else if (status == 403) {
					setStatus(document, "The subscriber you entered has already opted out of receiving emails.", 'red');
					setResult(document, '', '');
			    } else if (status == 409) {
					promptForUpdate(user_name, password, emailAddress.toLowerCase(), listsAry, extraFlds);
			    } else {
					setStatus(document, "Error: Unknown error " + status + ". Please check your network connection.", 'red');
					setResult(document, '', '');
				}
			}

			// notify user
			setStatus(document, "Please wait while your contact updates...", 'black');
			setResult(document, '', '');

			// call API
			comm.addContact(emailAddress, listsAry, extraFlds, addContactHandler);
		}

        /**
		 * Programmatically ends the application _and_ dispatches an 'exiting'
		 * event -- unlike NativeApplication.nativeApplication.exit().
		 * @method
		 * @private
		 */
		function appExit() {
			if(!appExiting) {
				appExiting = true;
				var app = air.NativeApplication
					.nativeApplication;
				var exitingEvent = new air.Event(
				 	air.Event.EXITING, true, true);
				app.dispatchEvent(exitingEvent);
				app.exit();
			}
		}

		// hook clicks on the link in the status message to trigger login
		// used for when the status message shows 'click here to log in'
		function hookStatusLinkToLogin() {
			var link = document.getElementById('statusMsgLoginLink');
			var clickHandler = function() {
				if (!modalOpen)
					showSettings();
			}
			link.addEventListener('click', clickHandler, false);
		}
		
		// hook the X button at the top of the window
		// makes the button hide the UI but not close the application
		function hookCloseButton() {
			var link = document.getElementById('closeButton');
			var clickHandler = function() {
				if (!modalOpen)
					window.nativeWindow.visible = false;
			}
			link.addEventListener('click', clickHandler, false);
		}

		// hook the settings button at the top of the window
		// makes the button show the settings window
		function hookSettingsButton() {
			var link = document.getElementById('settingsButton');
			var clickHandler = function() {
				if (!modalOpen)
					showSettings();
			}
			link.addEventListener('click', clickHandler, false);
		}

		// hook the help button at the top of the window
		// makes the button open the faq page
		function hookHelpButton() {
			var link = document.getElementById('helpButton');
			var clickHandler = function() {
				air.navigateToURL(new air.URLRequest('http://www.constantcontact.com/ws/contactcapture/faq.jsp'));
			}
			link.addEventListener('click', clickHandler, false);
		}

		// hook the list selector so that when the user clicks on it, it shows the list selector window
		// except on the first click of the application since that will likely open the settings window or auto log in
		function hookListSel() {
			var link = document.getElementById('listInput2');
			var clickHandler = function(event) {
				if (!modalOpen && !firstClick)
					showListSel();
			}
			link.addEventListener('click', clickHandler, false);
		}

		// hook the add/remove link so that it shows the list selector window
		// except on the first click of the application since that will likely open the settings window or auto log in
		function hookAddRemoveLink() {
			var link = document.getElementById('addRemoveLink');
			var clickHandler = function(event) {
				if (!modalOpen && !firstClick)
					showListSel();
			}
			link.addEventListener('click', clickHandler, false);
		}

		// hook the details button to expand the UI
		// this button toggles the state of the UI to expanded or compacted
		function hookDetailsButton() {
			var clickHandler = function() {
				if (modalOpen)
					return;

				var btn = document.getElementById('detailsButton');
				var addBtn = document.getElementById('addButton');
				if (btn.src.indexOf('detailsicon-open') != -1) {  // is closed
					btn.src = 'app:/src/images/detailsicon-closed.gif';  // change details button
					document.body.style.background = "url(src/images/widget-bg2.png) no-repeat";  // change background
					window.nativeWindow.height = 535;  // resize window
					addBtn.style.top = '500px';  // move 'add' button
					document.getElementById('field1_type_edit').style.display = 'block';  // show top type menu (is initially hidden)
					document.getElementById('field1_type_btn').style.display = 'block';
				} else {  // is open
					btn.src = 'app:/src/images/detailsicon-open.gif';
					document.body.style.background = "url(src/images/widget-bg1.png) no-repeat";
					window.nativeWindow.height = 329;
					addBtn.style.top = '293px';
					document.getElementById('field1_type_edit').style.display = 'none';  // hide top type menu since it can be seen in collapsed state
					document.getElementById('field1_type_btn').style.display = 'none';
				}
			}
			var link = document.getElementById('detailsButton');
			link.addEventListener('click', clickHandler, false);
			var txt = document.getElementById('detailsText');
			txt.addEventListener('click', clickHandler, false);
		}
		
		// hook the Add button to add a contact or log in if not logged in yet
		function hookAddButton() {
			var form = document.getElementById('appMainUIForm');
			var clickHandler = function(event){
				event.preventDefault();
				event.stopPropagation();

				if (modalOpen)
					return;
				
				// check if add button is disabled
				if (document.getElementById('addButton').style.backgroundImage.indexOf('disabled') != -1)
					return;

				// need to log in first?
				if (document.getElementById('loginSuccess').value != 'true')
					showSettings();
				else
					addEmailAddress();
			}
			form.addEventListener('submit', clickHandler, false);
		}

		// helper function to check the user settings
		// username, password, permission confirmation checkbox
		// set setAdd to true to automatically enable or disable the Add button
		function checkSettings(setAdd) {
			if (setAdd)
				enableAddBtn(false);

			// check username and password are present
			var credentialsFail = false;
			var user_name = document.getElementById('userName').value;
			var password = document.getElementById('password').value;
			if (typeof(password) == 'undefined' || password == null || /^\s*$/.exec(password) ||
				typeof(user_name) == 'undefined' || user_name == null || /^\s*$/.exec(user_name)) {
				setStatus(document, "You need to enter your user name and password. <a id=\"statusMsgLoginLink\" href=\"#\">Click here</a> to log in.", 'black');
				setResult(document, '', '');
				hookStatusLinkToLogin();
				credentialsFail = true;
			}

			// check permission confirmation
			var pcFail = !checkPermConf(setAdd);
			
			if (credentialsFail && pcFail) {
				setStatus(document, "You need to <a id=\"statusMsgLoginLink\" href=\"#\">log in</a> and accept the Permission Confirmation Checklist.", 'black');
				setResult(document, '', '');
				hookStatusLinkToLogin();
			}

			if (credentialsFail || pcFail)
				return false;

			if (setAdd)
				enableAddBtn(true);
			return true;
		}

		// helper function to check the permission confirmation checkbox
		// set setAdd to true to automatically enable or disable the Add button
		function checkPermConf(setAdd) {
			var pc = document.getElementById('permConf').value;
			if (pc != 'true') {
				setStatus(document, "You need to accept the Permission Confirmation Checklist on the Compliance tab under Settings.", 'black');
				setResult(document, '', '');
				if (setAdd)
					enableAddBtn(false);
				return false;
			}
			return true;
		}

		// helper function to check whether the user's saved password has expired
		function checkPasswordExpiration() {
			if (document.getElementById('passwordSaveCheck').value == 'true') {
				var t = document.getElementById('passwordSaveTime').value;
				var d = new Date();
				var now = d.getTime();
				if (t && (now - t) > 14 * 24 * 60 * 60 * 1000) {  // 14 days
					document.getElementById('passwordSave').value = '';
					document.getElementById('password').value = '';
				}
			}
		}

		/**
		 * Makes application's controlls persistent, i.e., their value will
		 * persist over application shutdown.
		 * @method
		 * @private
		 */
		function hookPersistence() {
			persistenceProvider = new PersistenceProvider ();
			persistenceProvider.registerElement(document.getElementById('userName'));
			persistenceProvider.registerElement(document.getElementById('passwordSave'));
			persistenceProvider.registerElement(document.getElementById('passwordSaveTime'));
			persistenceProvider.registerElement(document.getElementById('passwordSaveCheck'));
			persistenceProvider.registerElement(document.getElementById('permConf'));
			persistenceProvider.registerElement(document.getElementById('startupCheck'));
			persistenceProvider.registerElement(document.getElementById('eulaVersion'));
			persistenceProvider.registerElement(document.getElementById('field1_type_edit'));
			persistenceProvider.registerElement(document.getElementById('field1_type_id'));
			persistenceProvider.registerElement(document.getElementById('field2_type_edit'));
			persistenceProvider.registerElement(document.getElementById('field2_type_id'));
			persistenceProvider.registerElement(document.getElementById('field3_type_edit'));
			persistenceProvider.registerElement(document.getElementById('field3_type_id'));
			persistenceProvider.registerElement(document.getElementById('field4_type_edit'));
			persistenceProvider.registerElement(document.getElementById('field4_type_id'));
			persistenceProvider.registerElement(document.getElementById('listInput'));
			persistenceProvider.registerElement(document.getElementById('posX'));
			persistenceProvider.registerElement(document.getElementById('posY'));
			persistenceProvider.restoreNow();

			document.getElementById('password').value = document.getElementById('passwordSave').value;

			// store for access by child windows
			window.persistenceProvider = persistenceProvider;
		}

		/**
		 * Adds a tray/dock icon to the aplication. The icon is brought from the
		 * /icon folders -- the nearest resolution is automatically picked up.
		 * @method
		 * @private 
		 */
        function addTrayOrDockIcon() {

        	// @resolve urls to all image files we need to load:
        	var root = air.File.applicationDirectory;
			var iconsFolder = root.resolvePath('src/icons');
			if (iconsFolder && iconsFolder.exists) {
				var iconFiles = iconsFolder.getDirectoryListing();
				var iconFilesOK = true;
				for(var i=0; i<iconFiles.length; i++) {
					var iconFile = iconFiles[i];
					if (!/\.gif$|\.png$|\.bmp$|\.jpg$|\jpeg$/
						.test(iconFile.url)) {
						iconFilesOK = false;
						break;
					}
				}

				// @collect bitmapData objects:
				if(iconFilesOK) {
					var bmpDataObjects = [];
					var completeHandler = function(event){
						var bitmap = event.target.loader.content;
						bmpDataObjects.push(bitmap.bitmapData);
						loadNext();
					}
					var ioErrorHandler = function(event){
					}
					var loadNext = function(){
						var iconFile = iconFiles.pop();
						if(iconFile) {
							var iconURL = iconFile.url;
							var request = new air.URLRequest(iconURL);
							loader.load(request);
						} else {
							finish();
						}
					}
					var loader = new air.Loader();
		            loader.contentLoaderInfo.addEventListener(
		            	air.Event.COMPLETE, completeHandler);
		            loader.contentLoaderInfo.addEventListener(
		            	air.IOErrorEvent.IO_ERROR,
		            	ioErrorHandler);
		            loadNext();
				}

				// @display the shell icon:
				var finish = function() {
					var win = air.NativeApplication
						.supportsSystemTrayIcon;
					var osx = air.NativeApplication
						.supportsDockIcon;
					if (win || osx) {
						var app = air.NativeApplication
							.nativeApplication;
						var icon = app.icon;
						icon.bitmaps = bmpDataObjects;
						addTrayOrDockIconMenu(icon);
					}
				}
			}
        }

        /**
         * Adds a tray/doc menu that will show up when the user clickes the
         * application's tray/doc icon.
         * @method
         * @private
         * @param iconObj { Object }
         * 		The tray or dock icon object that is to attach a menu.
         */
        function addTrayOrDockIconMenu (iconObj) {

        	// @define a generic menu item click handler:
        	var menuClickHandler = function(event){
        		var item = event.target;
        		var itemName = item.name;
        		var actionName = itemName+ '_action'; 
        		if(item.data[actionName]) {
        			var action = item.data[actionName];
        			if (action instanceof Function) {action()};
        		}
        	};

        	// @define the menu items:
        	var openItem = new air.NativeMenuItem();
        	openItem.name = "open_item";
        	openItem.label = "Add Contact";
        	openItem.data = {
        		'open_item_action': function(){
        			openUI();
        		}
        	}
        	var separatorItem1 = new air.NativeMenuItem(null, true);
        	var settingsItem = new air.NativeMenuItem();
        	settingsItem.name = "settings_item";
        	settingsItem.label = "Settings...";
        	settingsItem.data = {
        		'settings_item_action': function(){
        			showSettings();
        		}
        	}
        	var aboutItem = new air.NativeMenuItem();
        	aboutItem.name = "about_item";
        	aboutItem.label = "About...";
        	aboutItem.data = {
        		'about_item_action': function(){
        			showAbout();
        		}
        	}
        	var separatorItem = new air.NativeMenuItem(null, true);
        	var exitItem = new air.NativeMenuItem();
        	exitItem.name = "exit_item";
        	exitItem.label = "Exit";
        	exitItem.data = {
        		'exit_item_action': function() {
					document.getElementById('posX').value = window.nativeWindow.x;
					document.getElementById('posY').value = window.nativeWindow.y;
					persistenceProvider.saveNow();
					appExit();
				}
        	};

        	// @hook items and finish setup:
        	var menu = new air.NativeMenu();
        	menu.addItem(openItem);
        	menu.addItem(separatorItem1);
        	menu.addItem(settingsItem);
        	menu.addItem(aboutItem);
        	menu.addItem(separatorItem);
        	menu.addItem(exitItem);
        	menu.addEventListener (air.Event.SELECT, menuClickHandler);
        	air.NativeApplication.nativeApplication.icon.menu = menu;
			air.NativeApplication.nativeApplication.icon.tooltip = "Click to Add Contact";
        	air.NativeApplication.nativeApplication.icon.addEventListener('click', openUI, false);

        }

		// helper function to show the UI if it is hidden
		function openUI() {
			window.nativeWindow.visible = true;
			window.nativeWindow.activate();
		}

		// array of data types for the custom/extra fields
		var data_types = {
		  "titles":[{"name":"None", "value":-1} , {"name":"First Name", "value":"FirstName"},
					{"name":"Middle Name", "value":"MiddleName"} , {"name":"Last Name", "value":"LastName"},
					{"name":"Job Title", "value":"JobTitle"}, {"name":"Company Name", "value":"CompanyName"},
					{"name":"Work Phone", "value":'WorkPhone'}, {"name":"Home Phone", "value":"HomePhone"},
					{"name":"Address Line 1", "value":"Addr1"}, {"name":"Address Line 2", "value":"Addr2"},
					{"name":"Address Line 3", "value":"Addr3"}, {"name":"City", "value":"City"},
					{"name":"US State/CA Province", "value":"StateCode"}, {"name":"Other State/Province", "value":"StateName"},
					{"name":"Country", "value":"CountryCode"}, {"name":"Zip/Postal Code", "value":"PostalCode"},
					{"name":"Sub Zip/Postal Code", "value":"SubPostalCode"},
					{"name":"Email Type", "value":"EmailType"},
					{"name":"Custom Field 1", "value":"CustomField1"},{"name": "Custom Field 2", "value":"CustomField2"},
					{"name":"Custom Field 3", "value":"CustomField3"}, {"name":"Custom Field 4", "value":"CustomField4"},
					{"name":"Custom Field 5", "value":"CustomField5"}, {"name":"Custom Field 6", "value":"CustomField6"},
					{"name":"Custom Field 7", "value":"CustomField7"}, {"name":"Custom Field 8", "value":"CustomField8"},
					{"name":"Custom Field 9", "value":"CustomField9"}, {"name":"Custom Field 10", "value":"CustomField10"},
					{"name":"Custom Field 11", "value":"CustomField11"}, {"name":"Custom Field 12", "value":"CustomField12"},
					{"name":"Custom Field 13", "value":"CustomField13"}, {"name":"Custom Field 14", "value":"CustomField14"},
					{"name":"Custom Field 15", "value":"CustomField15"} ]
		}

		// array of state codes for the state code selector
		var state_codes = {
		  "titles":[
{"name":"", "value":-1},
{"name":"None", "value":""},
{"name":"Alabama", "value":"AL"},
{"name":"Alaska", "value":"AK"},
{"name":"Alberta", "value":"AB"},
{"name":"Arizona", "value":"AZ"},
{"name":"Arkansas", "value":"AR"},
{"name":"Armed Forces Americas", "value":"AA"},
{"name":"Armed Forces Europe", "value":"AE"},
{"name":"Armed Forces Pacific", "value":"AP"},
{"name":"British Columbia", "value":"BC"},
{"name":"California", "value":"CA"},
{"name":"Colorado", "value":"CO"},
{"name":"Connecticut", "value":"CT"},
{"name":"Delaware", "value":"DE"},
{"name":"District of Columbia", "value":"DC"},
{"name":"Florida", "value":"FL"},
{"name":"Georgia", "value":"GA"},
{"name":"Hawaii", "value":"HI"},
{"name":"Idaho", "value":"ID"},
{"name":"Illinois", "value":"IL"},
{"name":"Indiana", "value":"IN"},
{"name":"Iowa", "value":"IA"},
{"name":"Kansas", "value":"KS"},
{"name":"Kentucky", "value":"KY"},
{"name":"Louisiana", "value":"LA"},
{"name":"Maine", "value":"ME"},
{"name":"Manitoba", "value":"MB"},
{"name":"Maryland", "value":"MD"},
{"name":"Massachusetts", "value":"MA"},
{"name":"Michigan", "value":"MI"},
{"name":"Minnesota", "value":"MN"},
{"name":"Mississippi", "value":"MS"},
{"name":"Missouri", "value":"MO"},
{"name":"Montana", "value":"MT"},
{"name":"Nebraska", "value":"NE"},
{"name":"Nevada", "value":"NV"},
{"name":"New Brunswick", "value":"NB"},
{"name":"New Hampshire", "value":"NH"},
{"name":"New Jersey", "value":"NJ"},
{"name":"New Mexico", "value":"NM"},
{"name":"New York", "value":"NY"},
{"name":"Newfoundland and Labrador", "value":"NL"},
{"name":"North Carolina", "value":"NC"},
{"name":"North Dakota", "value":"ND"},
{"name":"Northwest Territories", "value":"NT"},
{"name":"Nova Scotia", "value":"NS"},
{"name":"Nunavut", "value":"NU"},
{"name":"Ohio", "value":"OH"},
{"name":"Oklahoma", "value":"OK"},
{"name":"Ontario", "value":"ON"},
{"name":"Oregon", "value":"OR"},
{"name":"Pennsylvania", "value":"PA"},
{"name":"Prince Edward Island", "value":"PE"},
{"name":"Quebec", "value":"QC"},
{"name":"Rhode Island", "value":"RI"},
{"name":"Saskatchewan", "value":"SK"},
{"name":"South Carolina", "value":"SC"},
{"name":"South Dakota", "value":"SD"},
{"name":"Tennessee", "value":"TN"},
{"name":"Texas", "value":"TX"},
{"name":"Utah", "value":"UT"},
{"name":"Vermont", "value":"VT"},
{"name":"Virginia", "value":"VA"},
{"name":"Washington", "value":"WA"},
{"name":"West Virginia", "value":"WV"},
{"name":"Wisconsin", "value":"WI"},
{"name":"Wyoming", "value":"WY"},
{"name":"Yukon Territory", "value":"YT"}]
}

		// array of country codes for the country code selector
		var country_codes = {
		  "titles":[
{"name":"", "value":-1},
{"name":"None", "value":""},
{"name":"Afghanistan", "value":"af"},
{"name":"Aland Islands", "value":"ax"},
{"name":"Albania", "value":"al"},
{"name":"Algeria", "value":"dz"},
{"name":"American Samoa", "value":"as"},
{"name":"Andorra", "value":"ad"},
{"name":"Angola", "value":"ao"},
{"name":"Anguilla", "value":"ai"},
{"name":"Antarctica", "value":"aq"},
{"name":"Antigua and Barbuda", "value":"ag"},
{"name":"Argentina", "value":"ar"},
{"name":"Armenia", "value":"am"},
{"name":"Aruba", "value":"aw"},
{"name":"Australia", "value":"au"},
{"name":"Austria", "value":"at"},
{"name":"Azerbaijan", "value":"az"},
{"name":"Bahamas", "value":"bs"},
{"name":"Bahrain", "value":"bh"},
{"name":"Bangladesh", "value":"bd"},
{"name":"Barbados", "value":"bb"},
{"name":"Belarus", "value":"by"},
{"name":"Belgium", "value":"be"},
{"name":"Belize", "value":"bz"},
{"name":"Benin", "value":"bj"},
{"name":"Bermuda", "value":"bm"},
{"name":"Bhutan", "value":"bt"},
{"name":"Bolivia", "value":"bo"},
{"name":"Bosnia and Herzegovina", "value":"ba"},
{"name":"Botswana", "value":"bw"},
{"name":"Bouvet Island", "value":"bv"},
{"name":"Brazil", "value":"br"},
{"name":"British Indian Ocean Territory", "value":"io"},
{"name":"Brunei Darussalam", "value":"bn"},
{"name":"Bulgaria", "value":"bg"},
{"name":"Burkina Faso", "value":"bf"},
{"name":"Burundi", "value":"bi"},
{"name":"Cambodia", "value":"kh"},
{"name":"Cameroon", "value":"cm"},
{"name":"Canada", "value":"ca"},
{"name":"Cape Verde", "value":"cv"},
{"name":"Cayman Islands", "value":"ky"},
{"name":"Central African Republic", "value":"cf"},
{"name":"Chad", "value":"td"},
{"name":"Chile", "value":"cl"},
{"name":"China", "value":"cn"},
{"name":"Christmas Island", "value":"cx"},
{"name":"Cocos (Keeling) Islands", "value":"cc"},
{"name":"Colombia", "value":"co"},
{"name":"Comoros", "value":"km"},
{"name":"Congo", "value":"cg"},
{"name":"Congo, Democratic Republic of", "value":"cd"},
{"name":"Cook Islands", "value":"ck"},
{"name":"Costa Rica", "value":"cr"},
{"name":"Cote D'Ivoire", "value":"ci"},
{"name":"Croatia", "value":"hr"},
{"name":"Cyprus", "value":"cy"},
{"name":"Czech Republic", "value":"cz"},
{"name":"Denmark", "value":"dk"},
{"name":"Djibouti", "value":"dj"},
{"name":"Dominica", "value":"dm"},
{"name":"Dominican Republic", "value":"do"},
{"name":"East Timor", "value":"tmp"},
{"name":"Ecuador", "value":"ec"},
{"name":"Egypt", "value":"eg"},
{"name":"El Salvador", "value":"sv"},
{"name":"England", "value":"eng"},
{"name":"Equatorial Guinea", "value":"gq"},
{"name":"Eritrea", "value":"er"},
{"name":"Estonia", "value":"ee"},
{"name":"Ethiopia", "value":"et"},
{"name":"Faroe Islands", "value":"fo"},
{"name":"Faukland Islands", "value":"fk"},
{"name":"Fiji", "value":"fj"},
{"name":"Finland", "value":"fi"},
{"name":"France", "value":"fr"},
{"name":"French Guyana", "value":"gf"},
{"name":"French Polynesia", "value":"pf"},
{"name":"French Southern Territories", "value":"tf"},
{"name":"Gabon", "value":"ga"},
{"name":"Gambia", "value":"gm"},
{"name":"Georgia", "value":"ge"},
{"name":"Germany", "value":"de"},
{"name":"Ghana", "value":"gh"},
{"name":"Gibraltar", "value":"gi"},
{"name":"Greece", "value":"gr"},
{"name":"Greenland", "value":"gl"},
{"name":"Grenada", "value":"gd"},
{"name":"Guadeloupe", "value":"gp"},
{"name":"Guam", "value":"gu"},
{"name":"Guatemala", "value":"gt"},
{"name":"Guernsey", "value":"gg"},
{"name":"Guinea", "value":"gn"},
{"name":"Guine", "value":"issau - gw"},
{"name":"Guyana", "value":"gy"},
{"name":"Haiti", "value":"ht"},
{"name":"Heard and McDonald Islands", "value":"hm"},
{"name":"Honduras", "value":"hn"},
{"name":"Hong Kong", "value":"hk"},
{"name":"Hungary", "value":"hu"},
{"name":"Iceland", "value":"is"},
{"name":"India", "value":"in"},
{"name":"Indonesia", "value":"id"},
{"name":"Iraq", "value":"iq"},
{"name":"Ireland", "value":"ie"},
{"name":"Isle of Man", "value":"im"},
{"name":"Israel", "value":"il"},
{"name":"Italy", "value":"it"},
{"name":"Jamaica", "value":"jm"},
{"name":"Japan", "value":"jp"},
{"name":"Jersey", "value":"je"},
{"name":"Jordan", "value":"jo"},
{"name":"Kazakhstan", "value":"kz"},
{"name":"Kenya", "value":"ke"},
{"name":"Kiribati", "value":"ki"},
{"name":"Kuwait", "value":"kw"},
{"name":"Kyrgyzstan", "value":"kg"},
{"name":"Laos", "value":"la"},
{"name":"Latvia", "value":"lv"},
{"name":"Lebanon", "value":"lb"},
{"name":"Lesotho", "value":"ls"},
{"name":"Liberia", "value":"lr"},
{"name":"Libya", "value":"ly"},
{"name":"Liechtenstein", "value":"li"},
{"name":"Lithuania", "value":"lt"},
{"name":"Luxembourg", "value":"lu"},
{"name":"Macao", "value":"mo"},
{"name":"Macedonia", "value":"mk"},
{"name":"Madagascar", "value":"mg"},
{"name":"Malawi", "value":"mw"},
{"name":"Malaysia", "value":"my"},
{"name":"Maldives", "value":"mv"},
{"name":"Mali", "value":"ml"},
{"name":"Malta", "value":"mt"},
{"name":"Marshall Islands", "value":"mh"},
{"name":"Martinique", "value":"mq"},
{"name":"Mauritania", "value":"mr"},
{"name":"Mauritius", "value":"mu"},
{"name":"Mayotte", "value":"yt"},
{"name":"Mexico", "value":"mx"},
{"name":"Micronesia", "value":"fm"},
{"name":"Moldova", "value":"md"},
{"name":"Monaco", "value":"mc"},
{"name":"Mongolia", "value":"mn"},
{"name":"Montenegro", "value":"me"},
{"name":"Montserrat", "value":"ms"},
{"name":"Morocco", "value":"ma"},
{"name":"Mozambique", "value":"mz"},
{"name":"Myanmar", "value":"mm"},
{"name":"Namibia", "value":"na"},
{"name":"Nauru", "value":"nr"},
{"name":"Nepal", "value":"np"},
{"name":"Netherlands", "value":"nl"},
{"name":"Netherlands Antilles", "value":"an"},
{"name":"Neutral Zone", "value":"unavailable"},
{"name":"New Caledonia", "value":"nc"},
{"name":"New Zealand", "value":"nz"},
{"name":"Nicaragua", "value":"ni"},
{"name":"Niger", "value":"ne"},
{"name":"Nigeria", "value":"ng"},
{"name":"Niue", "value":"nu"},
{"name":"Norfolk Island", "value":"nf"},
{"name":"Northern Ireland", "value":"nir"},
{"name":"Northern Mariana Islands", "value":"mp"},
{"name":"Norway", "value":"no"},
{"name":"Oman", "value":"om"},
{"name":"Pakistan", "value":"pk"},
{"name":"Palau", "value":"pw"},
{"name":"Palestinian Territory, Occupied", "value":"ps"},
{"name":"Panama", "value":"pa"},
{"name":"Papua New Guinea", "value":"pg"},
{"name":"Paraguay", "value":"py"},
{"name":"Peru", "value":"pe"},
{"name":"Philippines", "value":"ph"},
{"name":"Pitcairn", "value":"pn"},
{"name":"Poland", "value":"pl"},
{"name":"Portugal", "value":"pt"},
{"name":"Puerto Rico", "value":"pr"},
{"name":"Qatar", "value":"qa"},
{"name":"Reunion", "value":"re"},
{"name":"Romania", "value":"ro"},
{"name":"Russian Federation", "value":"ru"},
{"name":"Rwanda", "value":"rw"},
{"name":"Saint Barthelemy", "value":"bl"},
{"name":"Saint Helena", "value":"sh"},
{"name":"Saint Kitts and Nevis", "value":"kn"},
{"name":"Saint Lucia", "value":"lc"},
{"name":"Saint Martin", "value":"mf"},
{"name":"Saint Pierre and Miquelon", "value":"pm"},
{"name":"Saint Vincent & the Grenadines", "value":"vc"},
{"name":"Samoa", "value":"ws"},
{"name":"San Marino", "value":"sm"},
{"name":"Sao Tome and Principe", "value":"st"},
{"name":"Saudi Arabia", "value":"sa"},
{"name":"Scotland", "value":"sct"},
{"name":"Senegal", "value":"sn"},
{"name":"Serbia", "value":"rs"},
{"name":"Seychelles", "value":"sc"},
{"name":"Sierra Leone", "value":"sl"},
{"name":"Singapore", "value":"sg"},
{"name":"Slovakia", "value":"sk"},
{"name":"Slovenia", "value":"si"},
{"name":"Solomon Islands", "value":"sb"},
{"name":"Somalia", "value":"so"},
{"name":"South Africa", "value":"za"},
{"name":"South Georgia & S. Sandwich Is.", "value":"gs"},
{"name":"South Korea", "value":"kr"},
{"name":"Spain", "value":"es"},
{"name":"Sri Lanka", "value":"lk"},
{"name":"Suriname", "value":"sr"},
{"name":"Svalbard and Jan Mayen", "value":"sj"},
{"name":"Swaziland", "value":"sz"},
{"name":"Sweden", "value":"se"},
{"name":"Switzerland", "value":"ch"},
{"name":"Taiwan", "value":"tw"},
{"name":"Tajikistan", "value":"tj"},
{"name":"Tanzania", "value":"tz"},
{"name":"Thailand", "value":"th"},
{"name":"Togo", "value":"tg"},
{"name":"Tokelau", "value":"tk"},
{"name":"Tonga", "value":"to"},
{"name":"Trinidad and Tobago", "value":"tt"},
{"name":"Tunisia", "value":"tn"},
{"name":"Turkey", "value":"tr"},
{"name":"Turkmenistan", "value":"tm"},
{"name":"Turks and Caicos Islands", "value":"tc"},
{"name":"Tuvalu", "value":"tv"},
{"name":"Uganda", "value":"ug"},
{"name":"Ukraine", "value":"ua"},
{"name":"United Arab Emirates", "value":"ae"},
{"name":"United Kingdom", "value":"gb"},
{"name":"United States", "value":"us"},
{"name":"United States Minor Outlying Is.", "value":"um"},
{"name":"Uruguay", "value":"uy"},
{"name":"Uzbekistan", "value":"uz"},
{"name":"Vanuatu", "value":"vu"},
{"name":"Vatican City State", "value":"va"},
{"name":"Venezuela", "value":"ve"},
{"name":"Viet Nam", "value":"vn"},
{"name":"Virgin Islands, British", "value":"vg"},
{"name":"Virgin Islands, U.S.", "value":"vi"},
{"name":"Wales", "value":"unavailable"},
{"name":"Wallis and Futuna", "value":"wf"},
{"name":"Western Sahara", "value":"eh"},
{"name":"Yemen", "value":"ye"},
{"name":"Zambia", "value":"zm"},
{"name":"Zimbabwe", "value":"zw"}]
}

		var emailtype_codes = {
		  "titles":[
{"name":"", "value":-1},
{"name":"HTML", "value":"HTML"},
{"name":"Text", "value":"Text"}]
}

		// helper function to fill a html SELECT element from a list
		function fillSelect(el, values) {
			for (var i=0; i<values.titles.length; i++) {
				var name = values.titles[i].name;
				var id = values.titles[i].value;

				// add to list
				var option = el.ownerDocument.createElement('option');
				option.appendChild(el.ownerDocument.createTextNode(name));
				option.setAttribute('value', id);
				el.appendChild(option);
			}
		}

		// helper function to fill an AIR menu from a list
		// n and nm are fieldNum and fieldName data to be stored with each menu option
		function fillMenu(rootMenu, values, n, nm) {
			var onItem = function(e) {
				var menuItem = e.target;
				document.getElementById('field' + menuItem.data['fieldNum'] + '_' + menuItem.data['fieldName'] + '_edit').value = menuItem.label;
				document.getElementById('field' + menuItem.data['fieldNum'] + '_' + menuItem.data['fieldName'] + '_id').value = menuItem.name;
				if (menuItem.data['fieldName'] == 'type')
					typechange(menuItem.data['fieldNum']);
			}

			for (var i=0; i<values.titles.length; i++) {
				var name = values.titles[i].name;
				var id = values.titles[i].value;

				// add to list
				var menuItem = new air.NativeMenuItem(name, false);
				menuItem.name = id;
				menuItem.data = { 'fieldNum' : n, 'fieldName' : nm }
				menuItem.addEventListener(air.Event.SELECT, onItem); 
				rootMenu.addItem(menuItem);
			}
		}

		/**
		 * Opens the list selection window
		 * so that the user can select contact lists
		 */
		function showListSel() {
			checkPasswordExpiration();

			// prevent opening duplicate if clicked again
			if (listSelWindow && listSelWindow == 1)
				return;

			if (listSelWindow && listSelWindow.window && listSelWindow.window.nativeWindow && !listSelWindow.window.nativeWindow.closed) {
				// activate existing window
				listSelWindow.window.nativeWindow.activate();
			} else {
				listSelWindow = 1;  // non-null value to prevent opening a duplicate before lists are loaded

				// close handler
				var listSelCloseHandler = function() {
					// invalidate
					listSelWindow = null;
					modalOpen = false;
					
					updateLists(document);
				}
				
				var refreshHandler = function() {
					// create new window
					var options = new air.NativeWindowInitOptions();
					options.systemChrome = "standard";
					options.type = "utility";
					options.resizable = false;
					var windowBounds = new air.Rectangle(200,200,300,470);
					listSelWindow = air.HTMLLoader.createRootWindow(
					   true, options, true, windowBounds);
					listSelWindow.addEventListener(air.Event.COMPLETE, function (e) {e.target.window.makeWindowModal()} );
					listSelWindow.load(new air.URLRequest("listsel.html"));
					listSelWindow.window.opener = window;
					listSelWindow.window.nativeWindow.addEventListener('close', listSelCloseHandler, false);
					modalOpen = true;
				}

				refreshLists(refreshHandler);
			}
		}
		
		/**
		 * Opens the settings window
		 * so that the user can log in, etc
		 */
		function showSettings() {
			checkPasswordExpiration();

			if (settingsWindow && settingsWindow.window && settingsWindow.window.nativeWindow && !settingsWindow.window.nativeWindow.closed) {
				// activate existing window
				settingsWindow.window.nativeWindow.activate();
			} else {
				// close handler
				var settingsCloseHandler = function() {
					// invalidate
					settingsWindow = null;
					
					modalOpen = false;
					
					// clear list if we're not logged in
					if (document.getElementById('loginSuccess').value != 'true') {
						clearList(document.getElementById('listInput'));
						clearList(document.getElementById('listInput2'));
					}

					checkSettings(true);
				}

				// create new window
				var options = new air.NativeWindowInitOptions();
				options.systemChrome = "standard";
				options.type = "utility";
				options.resizable = false;
				var windowBounds = new air.Rectangle(200,200,400,610);
				settingsWindow = air.HTMLLoader.createRootWindow(
				   true, options, true, windowBounds);
				settingsWindow.addEventListener(air.Event.COMPLETE, function (e) {e.target.window.makeWindowModal()} );
				settingsWindow.load(new air.URLRequest("settings.html"));
				settingsWindow.window.opener = window;
				settingsWindow.window.nativeWindow.addEventListener('close', settingsCloseHandler, false);
				modalOpen = true;
			}
		}

		/**
		 * Opens the EULA window
		 * so that the user can log in, etc
		 */
		function showEULA() {
			if (eulaWindow && eulaWindow.window && eulaWindow.window.nativeWindow && !eulaWindow.window.nativeWindow.closed) {
				// activate existing window
				eulaWindow.window.nativeWindow.activate();
			} else {
				// close handler
				var eulaCloseHandler = function() {
					// invalidate
					eulaWindow = null;
					
					modalOpen = false;
				}

				// create new window
				var options = new air.NativeWindowInitOptions();
				options.systemChrome = "standard";
				options.type = "utility";
				options.resizable = false;
				var windowBounds = new air.Rectangle(200,200,445,620);
				eulaWindow = air.HTMLLoader.createRootWindow(
				   true, options, true, windowBounds);
				eulaWindow.addEventListener(air.Event.COMPLETE, function (e) {e.target.window.makeWindowModal()} );
				eulaWindow.load(new air.URLRequest("eula.html"));
				eulaWindow.window.opener = window;
				eulaWindow.window.nativeWindow.addEventListener('close', eulaCloseHandler, false);
				modalOpen = true;
			}
		}

		/**
		 * Opens the about window
		 * to display application version and copyright
		 */
		function showAbout() {
			if (aboutWindow && aboutWindow.window && aboutWindow.window.nativeWindow && !aboutWindow.window.nativeWindow.closed) {
				// activate existing window
				aboutWindow.window.nativeWindow.activate();
			} else {
				// close handler
				var aboutCloseHandler = function() {
					aboutWindow = null;
					modalOpen = false;
				}
				
				// create new window
				var options = new air.NativeWindowInitOptions();
				options.systemChrome = "standard";
				options.type = "utility";
				options.resizable = false;
				var windowBounds = new air.Rectangle(200,250,300,100);
				aboutWindow = air.HTMLLoader.createRootWindow(
				   true, options, true, windowBounds);
				aboutWindow.addEventListener(air.Event.COMPLETE, function (e) {e.target.window.makeWindowModal()} );
				aboutWindow.load(new air.URLRequest("about.html"));
				aboutWindow.window.opener = window;
				aboutWindow.window.nativeWindow.addEventListener('close', aboutCloseHandler, false);
				modalOpen = true;
			}
		}

		/**
		 * Adds a custom css class to buttons to reflect their depressed state.
		 * @method
		 * @private
		 */
		function enhanceButtons () {
			var allInputs = document.getElementsByTagName('input');
			var buttonDown = function (event) {
				event.target.className = event.target.className+' down';
			}
			var buttonUp = function (event) {				
				event.target.className = event.target.className
					.replace(/\s*down/, '');
				event.target.className = event.target.className+ ' justClicked';
			}
			var buttonOut = function (event) {
				var el = event.target; 
				window.setTimeout(function() {
					while(el.className.indexOf('justClicked') != -1) {
						el.className =el.className.replace(/\s*justClicked/,'');
					}
				}, 10);
			}
			for(var i=0; i<allInputs.length; i++) {
				var input = allInputs[i];
				if (input.getAttribute('type') == 'button') {
					input.addEventListener('mousedown',	buttonDown,	false);
					input.addEventListener('mouseup',	buttonUp,	false);
					input.addEventListener('mouseout',	buttonOut,	false);
					input.addEventListener('mouseout',	buttonUp,	false);
				}
			}
		}

		// helper function to log the user into the constant contact API
		// this is done by loading the contact lists for the given constant contact user
		// this validates the username and password and also loads the list selector
		function doLogin() {
			var loadListsHandler = function(status, data) {
				if (status == 200) {
					var listsel = document.getElementById('listInput');
					fillList(data, listsel);
				} else if (status == 1000) {  // special 'done' status
					var listsel = document.getElementById('listInput');
					setStatus(document, 'Enter an email address and press Add.', 'black');
					setResult(document, '', '');
					document.getElementById('loginSuccess').value = 'true';
					// try to restore saved 
					if (listsel.selectedIndex == -1)
						persistenceProvider.restoreElement('listInput');
					// select General Interest if nothing is selected
					if (listsel.selectedIndex == -1) {
						for (var i=0; i<listsel.options.length; ++i) {
							if (listsel.options[i].value.indexOf('/lists/1') == (listsel.options[i].value.length - 8)) {  // match right at end of string
								listsel.options[i].selected = true;
								break;
							}
						}
					}
					updateLists(document);
					
					// select and de-select one item to prevent crash if user moves mouse from empty part of select to full part with button down (random AIR bug?)
					document.getElementById('listInput2').selectedIndex = 0;
					document.getElementById('listInput2').selectedIndex = -1;
				} else if (status == 401) {
					setStatus(document, "Please check your user name and password settings.", 'black');
					setResult(document, '', '');
					showSettings();
				} else if (status == 404) {
					setStatus(document, "The user name cannot contain spaces or characters such as '@'.", 'red');
					setResult(document, '', '');
					showSettings();
				} else if (status == 1001) {  // special parse error status
					setStatus(document, "Error parsing the data returned by the server. Please contact support.", 'red');
					setResult(document, '', '');
				} else {
					setStatus(document, "Error: Unknown error " + status + ". Please check your network connection.", 'red');
					setResult(document, '', '');
				}

				checkPermConf();
			}
		
			// load lists if data saved (in case login info changed)
			var user_name = document.getElementById('userName').value;
			var password = document.getElementById('password').value;
			document.getElementById('loginSuccess').value = 'false';
			setStatus(document, 'Logging in...', 'black');
			setResult(document, '', '');
			clearList(document.getElementById('listInput'));
			comm.loadLists(user_name, password, loadListsHandler);
		}

		// helper function to refresh the loaded contact lists
		// this function saves the selected lists first
		// so that the same lists will be selected after reloading
		function refreshLists(cbfn) {
			// save selected lists before reloading
			var listsel = document.getElementById('listInput');
			var a = [];
			var saved = false;
			if (listsel.selectedIndex != -1) {
				for (var i=0; i<listsel.options.length; ++i) {
					if (listsel.options[i].selected)
						a.push(listsel.options[i].value);
				}
				saved = true;
			}
		
			var loadListsHandler = function(status, data) {
				if (status == 200) {
					var listsel = document.getElementById('listInput');
					fillList(data, listsel);
				} else if (status == 1000) {  // special 'done' status
					var listsel = document.getElementById('listInput');
					
					// restore old list selections or saved selections
					if (saved) {
						for (var j=0; j<a.length; ++j) {
							for (var i=0; i<listsel.options.length; ++i) {
								if (listsel.options[i].value == a[j])
									listsel.options[i].selected = true;
							}
						}
					} else
						persistenceProvider.restoreElement('listInput');
		
					// select General Interest if nothing is selected
					if (listsel.selectedIndex == -1) {
						for (var i=0; i<listsel.options.length; ++i) {
							if (listsel.options[i].value.indexOf('/lists/1') == (listsel.options[i].value.length - 8)) {  // match right at end of string
								listsel.options[i].selected = true;
								break;
							}
						}
					}
		
					updateLists(document);
		
					setStatus(document, 'Enter an email address and press Add.', 'black');
					setResult(document, '', '');
					
					cbfn();
				} else if (status == 401) {
					setStatus(document, "Please check your user name and password settings.", 'red');
					setResult(document, '', '');
					alert("Please check your user name and password.");
				} else if (status == 404) {
					setStatus(document, "The user name cannot contain spaces or characters such as '@'.", 'red');
					setResult(document, '', '');
					alert("The user name cannot contain spaces or characters such as '@'.");
				} else if (status == 1001) {  // special parse error status
					setStatus(document, "Error parsing the data returned by the server. Please contact support.", 'red');
					setResult(document, '', '');
					alert('Error parsing the data returned by the server. Please contact support.');
				} else {
					setStatus(document, "Error: Unknown error. Please check your network connection.", 'red');
					setResult(document, '', '');
					alert('Error: Unknown error ' + status + '. Please check your network connection.');
				}
			}  // loadListsHandler()
		
			var user_name = document.getElementById('userName').value;
			var password = document.getElementById('password').value;
		
			// load lists if username and password are given
			if (user_name != '' && password != '') {
				clearList(listsel);
				setStatus(document, 'Loading lists...', 'black');
				setResult(document, '', '');
				comm.loadLists(user_name, password, loadListsHandler);
			} else {
				cbfn();
			}
		}

		/**
		 * The main class' entry point.
		 * @method
		 * @public
		 */
		this.run = function() {
			eventManager = EventManager.getDefault();
			enhanceUI();
			addTrayOrDockIcon();

			// make values auto-save
			hookPersistence();

			// hook UI button functionality
			hookAddButton();
			hookCloseButton();
			hookSettingsButton();
			hookHelpButton();
//			hookListSel();
			hookAddRemoveLink();
//			hookDetailsButton();

			// set extra fields to first name and last name the first time the app is run
			if (document.getElementById('field1_type_id').value == '') {
				document.getElementById('field1_type_id').value = 'FirstName';
				document.getElementById('field1_type_edit').value = 'First Name';
				document.getElementById('field2_type_id').value = 'LastName';
				document.getElementById('field2_type_edit').value = 'Last Name';
			}

			// restore last window position if saved
			var posX = document.getElementById('posX').value;
			var posY = document.getElementById('posY').value;
			if (posX)
				window.nativeWindow.x = posX;
			if (posY)
				window.nativeWindow.y = posY;

			// get version
			var xmlString = air.NativeApplication.nativeApplication.applicationDescriptor; 
			var appXml = new DOMParser(); 
			var xmlobject = appXml.parseFromString(xmlString, "text/xml"); 
			var root = xmlobject.getElementsByTagName('application')[0];
			var appVersion = root.getElementsByTagName("version")[0].firstChild.nodeValue; 
			
			// check version of EULA accepted
			if (appVersion != document.getElementById('eulaVersion').value) {
				showEULA();
			}

			// hook save of position on exit
			var onExit = function() {
				document.getElementById('posX').value = window.nativeWindow.x;
				document.getElementById('posY').value = window.nativeWindow.y;
				persistenceProvider.saveNow();
			}
			window.nativeWindow.addEventListener(window.air.Event.CLOSING, onExit);

			// allow clicks anywhere on the window (except the control elements) to move the window
			var onMove = function(evt) {
				var id = evt.target.id;
				if (id != 'listInput' && id != 'emailInput' && id != 'addButton' && id != 'closeButton' && id != 'settingsButton' &&
					id != 'detailsButton' && id != 'detailsText' &&
					id != 'field1' && id != 'field2' && id != 'field3' && id != 'field4' &&
					id != 'field1_state_edit' && id != 'field2_state_edit' &&	id != 'field3_state_edit' && id != 'field4_state_edit' &&
					id != 'field1_state_btn' && id != 'field2_state_btn' &&	id != 'field3_state_btn' && id != 'field4_state_btn' &&
					id != 'field1_country' && id != 'field2_country' && id != 'field3_country' && id != 'field4_country' && id != 'listInput2' &&
					id != 'field1_type_edit' && id != 'field2_type_edit' && id != 'field3_type_edit' && id != 'field4_type_edit' &&
					id != 'field1_type_btn' && id != 'field2_type_btn' && id != 'field3_type_btn' && id != 'field4_type_btn')
					window.nativeWindow.startMove();
			}
			document.body.addEventListener("mousedown", onMove, true);
			
			// allow first click anywhere on the window to activate the settings dialog if no password saved
			// or log in if password saved
			// and check for updates
			var onClick = function(evt) {
				if (firstClick) {
					firstClick = false;
					checkPasswordExpiration();
					var user_name = document.getElementById('userName').value;
					var password = document.getElementById('password').value;

					// check for updates					
					appUpdater.checkNow();
		
					// check settings
					if (typeof(password) == 'undefined' || password == null || /^\s*$/.exec(password) ||
						typeof(user_name) == 'undefined' || user_name == null || /^\s*$/.exec(user_name)) {
						showSettings();
						return;
					} else {
						doLogin();
					}
				}
			}
			document.body.addEventListener("click", onClick, false);

			// make sure utility windows (settings and about) are closed if main window is closed
			var appCloseHandler = function() {
				if (settingsWindow && settingsWindow.window && settingsWindow.window.nativeWindow && !settingsWindow.window.nativeWindow.closed)
					settingsWindow.window.nativeWindow.close();
				if (aboutWindow && aboutWindow.window && aboutWindow.window.nativeWindow && !aboutWindow.window.nativeWindow.closed)
					aboutWindow.window.nativeWindow.close();
				if (listSelWindow && listSelWindow.window && listSelWindow.window.nativeWindow && !listSelWindow.window.nativeWindow.closed)
					listSelWindow.window.nativeWindow.close();
			}
			window.nativeWindow.addEventListener('close', appCloseHandler, false);

			// set autostart option on first run
			var app = air.NativeApplication.nativeApplication;
			if (document.getElementById('startupCheck').value == '') {
				document.getElementById('startupCheck').value = 'true';
				// make app start on login/computer start
				try {
					app.startAtLogin = true;
				} catch (e) {
					alert("Error setting app to start automatically: " + e.message);
				}
			}

			// catch command line ivoke events to show UI
			// so that starting the application will cause the UI to come to the front if already started
			var onInvoke = function() {
				openUI();
			}
			app.addEventListener(air.InvokeEvent.INVOKE, onInvoke);

			// set initial status
			setStatus(document, 'Enter an email address and press Add.', 'black');
			setResult(document, '', '');
			
			// init comm interface
			comm = new APIComm();

			// update checker
			appUpdater = new runtime.air.update.ApplicationUpdaterUI(); 
			appUpdater.configurationFile = new air.File("app:/updateConfig.xml"); 
			appUpdater.initialize();

			// check password expiration
			checkPasswordExpiration();
			
			// check settings
			checkSettings(true);
			
			// set up type menus
			var typeMenu1 = new air.NativeMenu();
			fillMenu(typeMenu1, data_types, 1, 'type');
			var typeMenu2 = new air.NativeMenu();
			fillMenu(typeMenu2, data_types, 2, 'type');
			var typeMenu3 = new air.NativeMenu();
			fillMenu(typeMenu3, data_types, 3, 'type');
			var typeMenu4 = new air.NativeMenu();
			fillMenu(typeMenu4, data_types, 4, 'type');

			var typeMenuDisplay1 = function(event) {
				if (modalOpen)  return;
				typeMenu1.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var typeMenuDisplay2 = function(event) { 
				if (modalOpen)  return;
				typeMenu2.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var typeMenuDisplay3 = function(event) { 
				if (modalOpen)  return;
				typeMenu3.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var typeMenuDisplay4 = function(event) { 
				if (modalOpen)  return;
				typeMenu4.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var fld = document.getElementById('field1_type_edit');
			if (fld.value == '')  fld.value = 'None';
			fld.addEventListener('click', typeMenuDisplay1, false);
			fld = document.getElementById('field1_type_id');
			if (fld.value == '')  fld.value = '-1';
			fld = document.getElementById('field1_type_btn');
			fld.addEventListener('click', typeMenuDisplay1, false);

			fld = document.getElementById('field2_type_edit');
			if (fld.value == '')  fld.value = 'None';
			fld.addEventListener('click', typeMenuDisplay2, false);
			fld = document.getElementById('field2_type_id');
			if (fld.value == '')  fld.value = '-1';
			fld = document.getElementById('field2_type_btn');
			fld.addEventListener('click', typeMenuDisplay2, false);

			fld = document.getElementById('field3_type_edit');
			if (fld.value == '')  fld.value = 'None';
			fld.addEventListener('click', typeMenuDisplay3, false);
			fld = document.getElementById('field3_type_id');
			if (fld.value == '')  fld.value = '-1';
			fld = document.getElementById('field3_type_btn');
			fld.addEventListener('click', typeMenuDisplay3, false);

			fld = document.getElementById('field4_type_edit');
			if (fld.value == '')  fld.value = 'None';
			fld.addEventListener('click', typeMenuDisplay4, false);
			fld = document.getElementById('field4_type_id');
			if (fld.value == '')  fld.value = '-1';
			fld = document.getElementById('field4_type_btn');
			fld.addEventListener('click', typeMenuDisplay4, false);

			// set up state menus
			var stateMenu1 = new air.NativeMenu();
			fillMenu(stateMenu1, state_codes, 1, 'state');
			var stateMenu2 = new air.NativeMenu();
			fillMenu(stateMenu2, state_codes, 2, 'state');
			var stateMenu3 = new air.NativeMenu();
			fillMenu(stateMenu3, state_codes, 3, 'state');
			var stateMenu4 = new air.NativeMenu();
			fillMenu(stateMenu4, state_codes, 4, 'state');

			var stateMenuDisplay1 = function(event) { 
				if (modalOpen)  return;
				stateMenu1.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var stateMenuDisplay2 = function(event) { 
				if (modalOpen)  return;
				stateMenu2.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var stateMenuDisplay3 = function(event) { 
				if (modalOpen)  return;
				stateMenu3.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var stateMenuDisplay4 = function(event) { 
				if (modalOpen)  return;
				stateMenu4.display(window.nativeWindow.stage, event.clientX, 20); 
			}

			fld = document.getElementById('field1_state_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', stateMenuDisplay1, false);
			fld = document.getElementById('field1_state_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field1_state_btn');
			fld.addEventListener('click', stateMenuDisplay1, false);

			fld = document.getElementById('field2_state_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', stateMenuDisplay2, false);
			fld = document.getElementById('field2_state_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field2_state_btn');
			fld.addEventListener('click', stateMenuDisplay2, false);

			fld = document.getElementById('field3_state_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', stateMenuDisplay3, false);
			fld = document.getElementById('field3_state_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field3_state_btn');
			fld.addEventListener('click', stateMenuDisplay3, false);

			fld = document.getElementById('field4_state_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', stateMenuDisplay4, false);
			fld = document.getElementById('field4_state_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field4_state_btn');
			fld.addEventListener('click', stateMenuDisplay4, false);

			// set up country menus
			var countryMenu1 = new air.NativeMenu();
			fillMenu(countryMenu1, country_codes, 1, 'country');
			var countryMenu2 = new air.NativeMenu();
			fillMenu(countryMenu2, country_codes, 2, 'country');
			var countryMenu3 = new air.NativeMenu();
			fillMenu(countryMenu3, country_codes, 3, 'country');
			var countryMenu4 = new air.NativeMenu();
			fillMenu(countryMenu4, country_codes, 4, 'country');

			var countryMenuDisplay1 = function(event) { 
				if (modalOpen)  return;
				countryMenu1.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var countryMenuDisplay2 = function(event) { 
				if (modalOpen)  return;
				countryMenu2.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var countryMenuDisplay3 = function(event) { 
				if (modalOpen)  return;
				countryMenu3.display(window.nativeWindow.stage, event.clientX, 20); 
			}
			var countryMenuDisplay4 = function(event) { 
				if (modalOpen)  return;
				countryMenu4.display(window.nativeWindow.stage, event.clientX, 20); 
			}

			fld = document.getElementById('field1_country_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', countryMenuDisplay1, false);
			fld = document.getElementById('field1_country_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field1_country_btn');
			fld.addEventListener('click', countryMenuDisplay1, false);

			fld = document.getElementById('field2_country_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', countryMenuDisplay2, false);
			fld = document.getElementById('field2_country_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field2_country_btn');
			fld.addEventListener('click', countryMenuDisplay2, false);

			fld = document.getElementById('field3_country_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', countryMenuDisplay3, false);
			fld = document.getElementById('field3_country_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field3_country_btn');
			fld.addEventListener('click', countryMenuDisplay3, false);

			fld = document.getElementById('field4_country_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', countryMenuDisplay4, false);
			fld = document.getElementById('field4_country_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field4_country_btn');
			fld.addEventListener('click', countryMenuDisplay4, false);
			
			// set up email type menus
			var emailtypeMenu1 = new air.NativeMenu();
			fillMenu(emailtypeMenu1, emailtype_codes, 1, 'emailtype');
			var emailtypeMenu2 = new air.NativeMenu();
			fillMenu(emailtypeMenu2, emailtype_codes, 2, 'emailtype');
			var emailtypeMenu3 = new air.NativeMenu();
			fillMenu(emailtypeMenu3, emailtype_codes, 3, 'emailtype');
			var emailtypeMenu4 = new air.NativeMenu();
			fillMenu(emailtypeMenu4, emailtype_codes, 4, 'emailtype');

			var emailtypeMenuDisplay1 = function(event) { 
				if (modalOpen)  return;
				emailtypeMenu1.display(window.nativeWindow.stage, event.clientX, 430); 
			}
			var emailtypeMenuDisplay2 = function(event) { 
				if (modalOpen)  return;
				emailtypeMenu2.display(window.nativeWindow.stage, event.clientX, 478); 
			}
			var emailtypeMenuDisplay3 = function(event) { 
				if (modalOpen)  return;
				emailtypeMenu3.display(window.nativeWindow.stage, event.clientX, 526); 
			}
			var emailtypeMenuDisplay4 = function(event) { 
				if (modalOpen)  return;
				emailtypeMenu4.display(window.nativeWindow.stage, event.clientX, 575); 
			}

			fld = document.getElementById('field1_emailtype_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', emailtypeMenuDisplay1, false);
			fld = document.getElementById('field1_emailtype_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field1_emailtype_btn');
			fld.addEventListener('click', emailtypeMenuDisplay1, false);

			fld = document.getElementById('field2_emailtype_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', emailtypeMenuDisplay2, false);
			fld = document.getElementById('field2_emailtype_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field2_emailtype_btn');
			fld.addEventListener('click', emailtypeMenuDisplay2, false);

			fld = document.getElementById('field3_emailtype_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', emailtypeMenuDisplay3, false);
			fld = document.getElementById('field3_emailtype_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field3_emailtype_btn');
			fld.addEventListener('click', emailtypeMenuDisplay3, false);

			fld = document.getElementById('field4_emailtype_edit');
			if (fld.value == '')  fld.value = '';
			fld.addEventListener('click', emailtypeMenuDisplay4, false);
			fld = document.getElementById('field4_emailtype_id');
			if (fld.value == '')  fld.value = -1;
			fld = document.getElementById('field4_emailtype_btn');
			fld.addEventListener('click', emailtypeMenuDisplay4, false);
		}
	}
	context.instance = new _main();
	return context.instance;
}


